program casovani;

uses Classes, SysUtils, Generics.Collections;

type
  obsluhaCasovace = procedure of object;
  seznamObsluh = specialize TList<obsluhaCasovace>;

  TCasovac = class(TThread)
  private
    FInterval: integer;
    FObsluhy: seznamObsluh;
    FPovolen: Boolean;
    signalizace: PRTLEvent;
    procedure setPovolen(stav: Boolean);
    procedure Execute; override;
    procedure spustObsluhy;
  public
    Constructor Create(_interval: integer);
    property Interval: integer read FInterval write FInterval;
    property Povolen: Boolean read FPovolen write setPovolen;
    procedure pridejObsluhu(obsluha: obsluhaCasovace);
  end;

constructor TCasovac.Create(_interval: integer);
begin
   inherited Create(False);
   Interval:=_interval;
   FObsluhy := seznamObsluh.Create;
   FPovolen:=false;
   signalizace := RTLEventCreate;
end;

procedure TCasovac.setPovolen(stav: Boolean);
begin
   FPovolen:=stav;
   if FPovolen then
   begin
      RTLEventSetEvent(signalizace);
   end;
end;

procedure TCasovac.pridejObsluhu(obsluha: obsluhaCasovace);
begin
   FObsluhy.Add(obsluha);
end;

procedure TCasovac.spustObsluhy;
var i: integer;
begin
    for i := 0 to FObsluhy.Count - 1 do
    begin
      FObsluhy[i]();
    end;
end;

procedure TCasovac.Execute;
begin
   while true do
   begin
     Sleep(Interval);
     while not Povolen do
     begin
       RTLEventWaitFor(signalizace);
     end;
     spustObsluhy;
   end;
end;


type
  TPom = class
    procedure Tik;
  end;

procedure TPom.Tik;
begin
   writeln('Tik...');
end;

var
  casovac: TCasovac;
  p: TPom;


begin
  p := TPom.Create;
  casovac := TCasovac.Create(300);
  casovac.pridejObsluhu(@p.Tik);
  casovac.Povolen:=true;
  readln;
end.

